from __future__ import annotations  # for python 3.8


class MultiFile:
    name: str | None = None
    filename: str | None = None
    datafile: int | None = None
    length: int | None = None
    offset: int | None = None
